#pragma comment(lib, "SDL.lib")
#pragma comment(lib, "SDLmain.lib")
#pragma comment(lib, "SDL_ttf.lib")

#include "includeSdl.h"
#include <string.h>

#include "main.h"
#include "video.h"
#include "input.h"
#include "../pc/pcFS.h"

#include "../../emu/h/types.h"
#include "../../nes/h/nes.h"
#include "../../emu/h/emuPalette.h"
#include "../../emu/h/emuFileHandling.h"

#ifdef WIN32
#ifdef _DEBUG 
	#define _CRTDBG_MAP_ALLOC 
	#include <stdlib.h>
	#include <crtdbg.h> 
#endif
#endif

int appInit()
{
	if (initMemory())
		return 10;

	if (nesInit())
		return 11;

	if (initSound())
		return 12;

	if (initGUI())
		return 13;

	if (initInput())
		return 14;

	return 0;
}

void appDestory(void)
{
	uninitInput();
	uninitSound();
	nesClean();
	uninitMemory();
	destoryPalette();

	uninitFS();
	SDL_Quit();

	#ifdef WIN32
	#ifdef _DEBUG
		_CrtDumpMemoryLeaks(); 
	#endif
	#endif
}

char szFilter [] = ".nes\0";

int main(int argc, char *argv[])
{
	int i;
	SetLocalPath(argv[0]);

	if ( SDL_Init( SDL_INIT_EVERYTHING ) < 0 ) 
	{
		printf("Unable to init SDL: %s\n", SDL_GetError());
		SDL_Delay(2000);
		return 1;
	}

	atexit(appDestory);

	screen = SDL_SetVideoMode(resolutionWidth, resolutionHeight, resolutionDepth, SDL_SWSURFACE | SDL_ANYFORMAT | SDL_DOUBLEBUF | ( (argc > 1) ? SDL_FULLSCREEN : 0) );
	if ( screen == NULL ) 
	{
		printf("Unable to get screen (VideoMode): %s\n", SDL_GetError());
		return 2;
	}

	{
		char sz[300];
		strcpy( sz, argv[0] );
		strBack(sz);
		setFSFilter(szFilter);
		initFS(sz);
	}


	if (i = appInit())
	{
		printf("Error #%d, You seen the msg", i);
		SDL_Delay(2000);
		return 20;
	}

	{
		Uint8 ScreenDepth = screen->format->BitsPerPixel;

		if (ScreenDepth == 15)
			initPalette15();
		else if (ScreenDepth == 16)
			initPalette16();
		else if (ScreenDepth == 32)
			initPalette32();
		else
		{
			emuMsg("Invalid screen depth");
			return 5;		//unknown screen depth
		}
	}

	SDL_WM_SetCaption(wndName, 0);
	szFilename[0]=0;
	loadPath();
	mainLoop();
	return 0;
}
